# This file illustrates the default configuration values for Sprattus. All
# entries are optional and, if omitted, will take a default value same as in
# this file.

config.FunctionContext = {
    'AssumeNoUndef': False,     # perform analysis assuming that no undefined
                                # behavior occurs
    'RepresentGlobals': True,   # if false, global variables are not modelled
}

config.MemoryModel = {
    'Variant': 'NoMemory',
#   'Variant': 'LittleEndian',
#   'Variant': 'Aligned',
#   'Variant': 'BlockModel',
    'AddressBits': -1
}

config.FragmentDecomposition = {
    'Strategy': 'Edges',         # separate transformer for every CFG edges
#   'Strategy': 'Function',      # single transformer for the whole function;
                                 # CFG must be acyclic
#   'Strategy': 'Headers',       # abstraction points in loop headers
#   'Strategy': 'Body',          # abstraction points near the end of the loop
#   'Strategy': 'Backedges',     # abstract at the end of the loop and at the
                                 # header
}

config.Analyzer = {
    'Variant': 'UnilateralAnalyzer',
#   'Variant': 'BilateralAnalyzer',
    'Incremental': True,     # enables incremental SMT computation (if available)
    'WideningDelay': 20,     # how many iterations to wait before enabling widening
    'WideningFrequency': 10, # widen every `WideningFrequency` iterations
}

# Run `spranalyze -list-domains` to see the full list of supported domains with
# their documentation.
config.AbstractDomain = [SimpleConstProp, Equality(ParamStrategy.AllValuePairs(True))]

# Configuration for the Sprattus LLVM pass.
config.SprattusPass = {
    # enable verbose output
    'Verbose': False,

    # replace valus proven to be constant by the analysis with corresponding
    # constants
    'ConstantPropagation': True,

    # replace values discovered to be equivalent by the relational analysis
    'RedundantComputationRemoval': False,
}
