#pragma once

#include "utils.h"

namespace boost
{
namespace python
{
class dict;
}
}

namespace sprattus
{
namespace configparser
{
/**
 * Provides configuration from a Python file.
 */
class Config
{
  private:
    static constexpr const char* ENV_VAR = "SPRATTUS_CONFIG";

    shared_ptr<boost::python::dict> ConfigDict_;
    shared_ptr<boost::python::dict>
    tryLoadingFrom(const std::string& file_name);

  public:
    /**
     * Creates a `Config` object based on a given Python file.
     */
    Config(llvm::StringRef file_name);

    /**
     * Creates a `Config` object based on reasonable defaults.
     *
     * If the environmental variable `SPRATTUS_CONFIG` is set, it is assumed
     * that it contains a full path to a Python file containing the
     * configuration. Otherwise, the object is initialized using default values
     * of each option.
     */
    Config();

    template <typename T>
    T get(const char* module, const char* key, T default_value) const;
};
} // namespace configparser
} // namespace sprattus
