#pragma once

#include "utils.h"
#include "Config.h"
#include "ResultStore.h"

#include <set>
#include <z3++.h>
#include <llvm/Target/TargetLibraryInfo.h>

namespace llvm
{
class Function;
class CallInst;
class ReturnInst;
class DataLayout;
} // namespace llvm

namespace sprattus
{
class Analyzer;
class FunctionContext;

class ModuleContext
{
  private:
    llvm::Module* Module_;
    configparser::Config Config_;
    mutable unique_ptr<z3::context> Z3Context_;
    unique_ptr<ResultStore> Store_;
    mutable std::set<llvm::Function*> RecurFuncs_;
    unique_ptr<llvm::DataLayout> DataLayout_;
    unique_ptr<llvm::TargetLibraryInfo> TLI_;

    z3::expr formulaForBuiltin(llvm::Function* function) const;
    z3::expr substituteReturn(z3::expr formula, ValueMapping vmap,
                              llvm::ReturnInst* ret) const;
    std::set<z3::symbol> getSharedSymbols(FunctionContext* fctx) const;

  public:
    static std::string readGlobalString(llvm::Module* module, const char* name);

    ModuleContext(llvm::Module* module, configparser::Config config);

    z3::expr formulaFor(llvm::Function* function) const;

    unique_ptr<FunctionContext> createFunctionContext(llvm::Function* f) const;

    ResultStore* getResultStore() const { return Store_.get(); }
    llvm::DataLayout* getDataLayout() const { return DataLayout_.get(); }
    llvm::TargetLibraryInfo* getTargetLibraryInfo() const { return TLI_.get(); }

    z3::context& getZ3() const { return *Z3Context_; }
    configparser::Config getConfig() const { return Config_; }

    z3::symbol getReturnSymbol() const
    {
        return Z3Context_->str_symbol("__RETURN__");
    }
};
} // namespace sprattus
