/**
 * Runtime support for dynamic analyses.
 *
 * Can be used from C code.
 */

#ifndef SPRATTUS_RUNTIME_H
#define SPRATTUS_RUNTIME_H

#include <stdint.h>

#ifdef __cplusplus
#include "utils.h"
#include "Config.h"
#include "ResultStore.h"
#include "ConcreteState.h"

namespace sprattus
{
class Runtime
{
  private:
    static constexpr unsigned CACHE_SIZE = 524288;

    struct CacheEntry {
        uint32_t location_id;
        unique_ptr<AbstractValue> value;
    };

    configparser::Config GlobalConfig_;
    unique_ptr<llvm::Module> Module_;
    unique_ptr<ModuleContext> Mctx_;
    std::vector<FunctionContext>
        FunctionContextV_; // has to be declared before ResultStore
    ResultStore ResultStore_;
    CacheEntry Cache_[CACHE_SIZE];
    std::vector<ConcreteState::Value> ConcreteValues_;

    Runtime();
    void evict(CacheEntry* entry);

  public:
    /**
     * Environmental variable used to pass the database file name.
     */
    static constexpr const char* ENV_VAR = "SPRATTUS_DB";

    static Runtime* get()
    {
        static Runtime runtime;
        return &runtime;
    }

    void trace(uint32_t function_id, uint32_t location_id,
               const ConcreteState& cstate);

    const FunctionContext& contextFor(uint32_t function_id)
    {
        return FunctionContextV_[function_id];
    }

    /**
     * Returns a pointer to a continuous block of memory big enough to hold
     * all the represented values in any given function.
     *
     * The memory is managed by the Runtime instance.
     */
    ConcreteState::Value* valueCache() { return &ConcreteValues_[0]; }

    ~Runtime();
};
} // namespace sprattus

extern "C" {
#endif

void sprattus_trace(uint32_t function_id, uint32_t location_id, ...);
void sprattus_probe(int ignored, ...);

extern const char sprattus_rt_database_path[];
extern const char sprattus_rt_bitcode_path[];
extern const char sprattus_rt_config[];

#ifdef __cplusplus
} // extern "C"
#endif

#endif
