#include "utils.h"
#include "repr.h"
#include "AbstractValue.h"
#include "DomainConstructor.h"
#include "ParamStrategy.h"
#include "domains/Product.h"
#include "Config.h"

#include <boost/python.hpp>

using namespace sprattus;
using namespace sprattus::configparser;
using namespace boost::python;
namespace py = boost::python;

namespace // unnamed
{
template <typename T> list iterable_to_list(const T& iterable)
{
    list result;
    for (auto& x : iterable) {
        result.append(x);
    }
    return result;
}

list _DomainConstructor_all()
{
    return iterable_to_list(DomainConstructor::all());
}

DomainConstructor _DomainConstructor_product(list doms)
{
    std::vector<DomainConstructor> input(len(doms));
    for (int i = 0; i < len(doms); i++) {
        input[i] = py::extract<DomainConstructor>(doms[i]);
    }
    return DomainConstructor::product(input);
}

template <ParamStrategy (*F)(bool)> ParamStrategy default_false()
{
    return F(false);
}

DomainConstructor _DomainConstructor_makeCustom(std::string name,
                                                std::string description,
                                                int arity, py::object func)
{
    assert(PyCallable_Check(func.ptr()));

    auto f = [arity, func](const DomainConstructor::args& args) {
        try {
            assert((int)args.parameters.size() == arity);

            py::tuple func_args(iterable_to_list(args.parameters));
            py::object func_res(
                handle<>(PyObject_CallObject(func.ptr(), func_args.ptr())));

            py::extract<DomainConstructor> xtr(func_res);
            return DomainConstructor(xtr).makeBottom(args);
        } catch (py::error_already_set&) {
            PyErr_PrintEx(1);
            panic("Invalid Python DomainConstructor implementation");
        }
    };

    return DomainConstructor(name, description, arity, f);
}

std::string _Expression_repr(Expression e)
{
    return "<Expression " + repr(e) + ">";
}
} // namespace unnamed

BOOST_PYTHON_MODULE(libsprattus)
{
    class_<DomainConstructor>("DomainConstructor", no_init)
        .def("all", &_DomainConstructor_all)
        .staticmethod("all")

        .def("product", &_DomainConstructor_product)
        .staticmethod("product")

        .def("_makeCustom", &_DomainConstructor_makeCustom)
        .staticmethod("_makeCustom")

        .add_property(
             "name", make_function(&DomainConstructor::name,
                                   return_value_policy<copy_const_reference>()))
        .add_property(
             "description",
             make_function(&DomainConstructor::description,
                           return_value_policy<copy_const_reference>()))

        .add_property("arity", make_function(&DomainConstructor::arity))
        .def("__repr__", &repr<DomainConstructor>)
        .def("_parameterize", &DomainConstructor::parameterize);

    class_<ParamStrategy>("ParamStrategy", no_init)
        .def("FromExpression", &ParamStrategy::FromExpression)
        .staticmethod("FromExpression")

        .def("AllValues", &ParamStrategy::AllValues)
        .staticmethod("AllValues")

        .def("AllValuePairs", ParamStrategy::AllValuePairs)
        .def("AllValuePairs", &default_false<ParamStrategy::AllValuePairs>)
        .staticmethod("AllValuePairs")

        .def("PackedPairs", ParamStrategy::PackedPairs)
        .def("PackedPairs", &default_false<ParamStrategy::PackedPairs>)
        .staticmethod("PackedPairs")

        .def("ConstOffsets", ParamStrategy::ConstOffsets)
        .def("ConstOffsets", &default_false<ParamStrategy::ConstOffsets>)
        .staticmethod("ConstOffsets")

        .def("PointerPairs", ParamStrategy::PointerPairs)
        .def("PointerPairs", &default_false<ParamStrategy::PointerPairs>)
        .staticmethod("PointerPairs")

        .def("NonPointerPairs", ParamStrategy::NonPointerPairs)
        .def("NonPointerPairs", &default_false<ParamStrategy::NonPointerPairs>)
        .staticmethod("NonPointerPairs")

        .def("Pointers", &ParamStrategy::Pointers)
        .staticmethod("Pointers")

        .def("NonPointers", &ParamStrategy::NonPointers)
        .staticmethod("NonPointers")

        .add_property("arity", make_function(&ParamStrategy::arity))
        .def("__repr__", &repr<ParamStrategy>);

    class_<Expression>("Expression", no_init)
        .def("__repr__", &_Expression_repr);
}
