/**
 * RUN: clang -c -emit-llvm %s -o %t1.pre.bc
 * RUN: opt -mem2reg -instnamer %t1.pre.bc -o %t1.bc
 * RUN: %sprinstrument -config all_serializable_domains %t1.bc %t2.db %t2.bc
 * RUN: clang %t2.bc %rt_flags -o %t2
 * RUN: %t2
 * RUN: %spranalyze -mode=only-dynamic -where=everywhere -function url_decode %t2.bc | %probe_filter %t1.bc > %t2.post
 *
 * FIXME: Find some way to ensure that we're checking the right basic block.
 * RUN: grep "probe3 >= probe1" %t2.post
 * RUN: grep "probe2 >= probe1" %t2.post
 * RUN: grep "probe2 >= probe3" %t2.post
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>

extern void sprattus_probe(int, ...);

/**
 * Taken from OpenBSD's httpd.
 * Source: https://github.com/reyk/httpd/blob/master/httpd/httpd.c
 */
const char *
url_decode(char *url)
{
	char		*p, *q;
	char		 hex[3];
	unsigned long	 x;

	hex[2] = '\0';
	p = q = url;

	while (*p != '\0') {
		sprattus_probe(0, url, p, q);

		switch (*p) {
		case '%':
			/* Encoding character is followed by two hex chars */
			if (!(isxdigit(p[1]) && isxdigit(p[2])))
				return (NULL);

			hex[0] = p[1];
			hex[1] = p[2];

			/*
			 * We don't have to validate "hex" because it is
			 * guaranteed to include two hex chars followed by nul.
			 */
			x = strtoul(hex, NULL, 16);
			*q = (char)x;
			p += 2;
			break;
		default:
			*q = *p;
			break;
		}
		p++;
		q++;
	}
	*q = '\0';

	return (url);
}

int main(int argc, char** argv)
{
	char buf1[] = "Hello world%21%0D%0A";
	char buf2[] = "";
	char buf3[] = "%00";
	char buf4[] = "%";

	url_decode(buf1);
	url_decode(buf2);
	url_decode(buf3);
	url_decode(buf4);

	return 0;
}
