; The numerical relational domain can be also used to track simple logical
; relations between boolean (i1) variables. In particular, for 0-1 variables
; a and b, "a =< b" can be interpreted as an implication "a ==> b".
;
; RUN: opt %s > %t1.bc
;
; RUN: %spranalyze -function=implication -config=02_rel_function %t1.bc > %t1_implication.out
; RUN: grep -E 'a =< b|b >= a' %t1_implication.out
;
; RUN: %spranalyze -function=equivalence -config=02_rel_function %t1.bc > %t1_equivalence.out
; RUN: grep -E 'a = b|b = a' %t1_equivalence.out
;
; RUN: %spranalyze -function=exclusion -config=02_rel_function %t1.bc > %t1_exclusion.out
; RUN: grep -E 'a != b|b != a' %t1_exclusion.out


; (x <= 3) ==> (x <= 5)
define void @implication(i32 %x) #0 {
start:
  %a = icmp sle i32 %x, 3
  br i1 %a, label %a_true, label %a_false

a_true:
  br label %middle

a_false:
  br label %middle

middle:
  %b = icmp sle i32 %x, 5
  br i1 %b, label %b_true, label %b_false

b_true:
  br label %end

b_false:
  br label %end

end:
  ret void
}

; (x == 0) <==> (x + 3 == 3)
define void @equivalence(i32 %x) #0 {
start:
  %a = icmp eq i32 %x, 0
  br i1 %a, label %a_true, label %a_false

a_true:
  br label %middle

a_false:
  br label %middle

middle:
  %x_plus_three = add i32 %x, 3
  %b = icmp eq i32 %x_plus_three, 3
  br i1 %b, label %b_true, label %b_false

b_true:
  br label %end

b_false:
  br label %end

end:
  ret void
}

; (x < 0) <==> !(x >= 0)
define void @exclusion(i32 %x) #0 {
start:
  %a = icmp slt i32 %x, 0
  br i1 %a, label %a_true, label %a_false

a_true:
  br label %middle

a_false:
  br label %middle

middle:
  %b = icmp sge i32 %x, 0
  br i1 %b, label %b_true, label %b_false

b_true:
  br label %end

b_false:
  br label %end

end:
  ret void
}
