// RUN: clang -c -emit-llvm %s -o %t1.bc
// RUN: opt -always-inline -instcombine -mem2reg -instnamer %t1.bc -o %t2.bc
// RUN: %spranalyze -check-assertions -config=01_const_function %t2.bc > %t3

#include <stdint.h>
#include <assert.h>

#define STRAWBERRY (1L<<1)
#define APPLE      (1L<<0)
#define ORANGE     (1L<<32)
#define BANANA     (1L<<14)
#define AVOCADO    (1L<<7)

static uint8_t __attribute__((always_inline)) popcount(uint32_t x)
{
	const uint32_t MASK1  = 0x55555555; /* 0x5 = 0b0101 */
	const uint32_t MASK2  = 0x33333333; /* 0x3 = 0b0011 */
	const uint32_t MASK4  = 0x0f0f0f0f;
	const uint32_t MASK8  = 0x00ff00ff;
	const uint32_t MASK16 = 0x0000ffff;

	x = (x & MASK1) + ((x & ~MASK1) >> 1);
	x = (x & MASK2) + ((x & ~MASK2) >> 2);
	x = (x & MASK4) + ((x & ~MASK4) >> 4);
	x = (x & MASK8) + ((x & ~MASK8) >> 8);
	x = (x & MASK16) + ((x & ~MASK16) >> 16);

	return x;
}

void test(uint8_t x)
{
	uint32_t fruit = BANANA | AVOCADO;

	if (x)
		fruit |= STRAWBERRY;
	else
		fruit |= APPLE;

	uint8_t res = popcount(fruit);
	assert(res == 3);
}

#ifdef CBMC
// can be verified with: cbmc -DCBMC <this_file>.c
int main()
{
	extern uint8_t nondet_uint8();
	test(nondet_uint8());
	return 0;
}
#endif
