// RUN: clang -c -emit-llvm %s -o %t1.bc
// RUN: opt -O3 -unroll-threshold=4294967295 -mem2reg -instnamer %t1.bc -o %t2.bc
// RUN: %spranalyze -check-assertions -config=01_const_function %t2.bc > %t3

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

static uint8_t __attribute__((always_inline)) popcount(uint32_t x)
{
	const uint32_t MASK1  = 0x55555555; /* 0x5 = 0b0101 */
	const uint32_t MASK2  = 0x33333333; /* 0x3 = 0b0011 */
	const uint32_t MASK4  = 0x0f0f0f0f;
	const uint32_t MASK8  = 0x00ff00ff;
	const uint32_t MASK16 = 0x0000ffff;

	x = (x & MASK1) + ((x & ~MASK1) >> 1);
	x = (x & MASK2) + ((x & ~MASK2) >> 2);
	x = (x & MASK4) + ((x & ~MASK4) >> 4);
	x = (x & MASK8) + ((x & ~MASK8) >> 8);
	x = (x & MASK16) + ((x & ~MASK16) >> 16);

	return x;
}

static uint8_t __attribute__((always_inline)) popcount_naive(uint32_t x)
{
	uint8_t result = 0;
	const uint32_t ONE = 1;

	for(int i = 0; i < 32; i++) {
		if (x & (ONE<<i))
			result++;
	}

	return result;
}

void test(uint32_t x)
{
	assert(popcount(x) == popcount_naive(x));
}

#ifdef CBMC
// can be verified with: cbmc -DCBMC <this_file>.c
int main(int argc, char** argv)
{
	extern uint32_t nondet_uint64();
	uint32_t x = nondet_uint64();
	test(x);
	return 0;
}
#endif
