#include "llvm/Pass.h"
#include "llvm/IR/Module.h"
#include "llvm/IR/Function.h"
#include "llvm/IR/Attributes.h"

using namespace llvm;

namespace
{
struct PrepPass : public ModulePass {
    static char ID;
    PrepPass() : ModulePass(ID) {}

    bool runOnModule(Module& M) override
    {
        bool changed = false;
        for (auto& f : M) {
            const auto& name = f.getName();
            if (name.startswith("__VERIFIER_nondet_")) {
                f.addFnAttr(Attribute::AttrKind::ReadNone);
                changed = true;
            }
        }
        return changed;
    }
};
} // end of anonymous namespace

char PrepPass::ID = 0;
static RegisterPass<PrepPass>
    X("prep", "Preparation Pass (for annotating SVCOMP16 intrinsics)",
      true /* Only looks at CFG */, false /* Analysis Pass */);
