#pragma once

#include <llvm/AsmParser/Parser.h>
#include <llvm/IR/Module.h>
#include <llvm/IR/LLVMContext.h>
#include <llvm/Support/ErrorHandling.h>
#include <llvm/Support/SourceMgr.h>
#include <llvm/Support/raw_ostream.h>

#include <gtest/gtest.h>

#include <tuple>

#include "utils.h"
#include "repr.h"

template <typename T>
std::vector<typename T::value_type*> pointerVector(const T& xs)
{
    std::vector<typename T::value_type*> result;
    for (auto& x : xs)
        result.push_back(const_cast<typename decltype(result)::value_type>(&x));
    return result;
}

std::unique_ptr<llvm::Module> parseAssembly(const char* assembly)
{
    llvm::SMDiagnostic error;

#if LLVM35
    std::unique_ptr<llvm::Module> module(
        new llvm::Module("Module", llvm::getGlobalContext()));

    bool success = ParseAssemblyString(assembly, module.get(), error,
                                       module->getContext()) == module.get();
    assert(success);
#else
    std::unique_ptr<llvm::Module> module =
        parseAssemblyString(assembly, error, llvm::getGlobalContext());
    assert(module);
#endif

    return module;
}

template <int n, typename TT, typename T> struct TieVector {
    void operator()(TT tie, const std::vector<T>& in)
    {
        std::get<n - 1>(tie) = in[n - 1];
        TieVector<n - 1, TT, T>()(tie, in);
    }
};

template <typename TT, typename T> struct TieVector<1, TT, T> {
    void operator()(TT tie, const std::vector<T>& in)
    {
        std::get<0>(tie) = in[0];
    }
};

template <typename TT, typename T> void unpack(TT tie, const std::vector<T>& in)
{
    constexpr int size = std::tuple_size<TT>::value;
    assert(size == in.size());
    TieVector<size, TT, T>()(tie, in);
}
